import { t } from '../../i18n'
import dateUtil from '../../utils/date'
import Utils from '../../utils/common'

const DEFAULT_FORMAT = 'yyyy-MM-dd'
const DEFAULT_FORMAT_WEEK = 'yyyywWW'
const DEFAULT_FORMAT_QUARTER = 'yyyyqQQ'

export const weeks = ['sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat'].map(day => t(`date.weeks.${day}`))
export const monthShort = ['jan', 'feb', 'mar', 'apr', 'may', 'jun', 'jul', 'aug', 'sep', 'oct', 'nov', 'dec']
const i18nProps = () => {
  return {
    dayNamesShort: weeks,
    dayNames: weeks,
    monthNamesShort: monthShort.map(month => t(`date.months.${ month }`)),
    monthNames: monthShort.map((ignore, index) => t(`date.month${ index + 1 }`)),
    amPm: ['am', 'pm']
  }
}

export const getDayCountOfMonth = (year, month) => {
  const months = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]
  if (month === 1) {
    if ((year % 400 === 0) || (year % 4 === 0 && year % 100 !== 0)) {
      return 29
    }
    return months[1]
  }
  return months[month]
}

const toDate = date => {
  return Utils.isDate(date) ? new Date(date) : null
}

export const parseDate = (string, format = DEFAULT_FORMAT) => {
  return dateUtil.parse(string, extractDateFormat(format), i18nProps())
}

export const formatDate = (date, format = DEFAULT_FORMAT) => {
  date = toDate(date)
  if (date) {
    return dateUtil.format(date, extractDateFormat(format), i18nProps())
  }
}

export const formatDateTime = (date, format = DEFAULT_FORMAT) => {
  date = toDate(date)
  if (date) {
    return dateUtil.format(date, format, i18nProps())
  }
}

const extractDateFormat = format => {
  return format.replace(/\W?m{1,2}|\W?ZZ/g, '').replace(/\W?h{1,2}|\W?s{1,3}|\W?a/gi, '').trim()
}

export const getArrayBtwTwoDates = (start, end, format = true) => {
  const curDate = new Date(start.getTime())
  const dates = []
  while (curDate <= end) {
    const date = new Date(curDate)
    dates.push(format ? formatDate(date) : date)
    curDate.setDate(curDate.getDate() + 1)
  }
  return dates
}

export const getPrevMonthDate = date => {
  const day = new Date(date.getFullYear(), date.getMonth(), 1)
  return getFirstDateOfMonth(day)
  // return new Date(date.getFullYear() - (date.getMonth() > 0 ? 0 : 1), (date.getMonth() - 1 + 12) % 12, 1)
}
export const getNextMonthDate = date => {
  return new Date(date.getFullYear(), date.getMonth() + 1, 1)
}
export const getPrevYearDate = date => {
  return new Date(date.getFullYear() - 1, date.getMonth())
}
export const getNextYearDate = date => {
  return new Date(date.getFullYear() + 1, date.getMonth())
}
export const getFirstDateOfMonth = date => {
  const curDate = new Date(date.getTime())
  curDate.setDate(1)
  curDate.setHours(0, 0, 0, 0)
  return curDate
}
export const getQuarter = date => {
  var quarters = [1, 2, 3, 4]
  return quarters[Math.floor(date.getMonth() / 3)]
}
export const getWeekNumber = date => {
  if (!Utils.isDate(date)) {
    return null
  }
  // https://stackoverflow.com/questions/36355907/how-to-get-week-number-in-js
  const curDate = new Date(date.getTime())
  curDate.setHours(0, 0, 0, 0)
  // Thursday in current week decides the year.
  curDate.setDate(curDate.getDate() + 3 - (curDate.getDay() + 6) % 7)
  // January 4 is always in week 1.
  const week1 = new Date(curDate.getFullYear(), 0, 4)
  // Adjust to Thursday in week 1 and count number of weeks from date to week 1.
  return 1 + Math.round(((curDate.getTime() - week1.getTime()) / 86400000 - 3 + (week1.getDay() + 6) % 7) / 7)
}
export const getFirstOfWeek = date => {
  const curDate = new Date(date.getTime())
  curDate.setHours(0, 0, 0, 0)
  // Set date to previous Sunday
  curDate.setDate(curDate.getDate() - curDate.getDay())
  return curDate
}
export const getLastOfWeek = date => {
  date = getFirstOfWeek(date)
  date.setDate(date.getDate() + 6)
  return date
}
export const formatWeek = (date, format = DEFAULT_FORMAT_WEEK) => {
  if (!date) {
    return
  }
  let week = getWeekNumber(date)
  let month = date.getMonth()
  const orgDate = new Date(date)
  if (week === 1 && month === 11) {
    orgDate.setHours(0, 0, 0, 0)
    orgDate.setDate(orgDate.getDate() + 3 - (orgDate.getDay() + 6) % 7)
  }
  date = formatDate(orgDate, format)
  date = /WW/.test(date) ? date.replace(/WW/, week < 10 ? `0${week}` : week) : date.replace(/W/, week)
  return date
}
export const formatQuarter = (date, format = DEFAULT_FORMAT_QUARTER) => {
  if (!date) {
    return
  }
  let quarter = getQuarter(date)
  date = formatDate(date, format)
  date = /QQ/.test(date) ? date.replace(/QQ/, `Q${quarter}`) : date.replace(/Q/, `Q${quarter}`)
  return date
}
export const parseWeek = text => {
  const array = (text || '').split('w')
  if (array.length === 2) {
    const year = Number(array[0])
    const month = Number(array[1])
    if (!isNaN(year) && !isNaN(month) && month < 54) {
      return text
    }
  }
  return null
}
